<?php

/**
 * @package     JTmap
 *
 * @copyright   Copyright (C) 2005 - 2023 jtotal.org All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt

 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

namespace Jtotal\Plugin\Content\Jtmap\Extension;

use Joomla\CMS\Categories\Categories;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;
use Jtotal\Jtfw\Helper\JtLicense;
use Jtotal\Plugin\Content\Jtmap\Helper\JtmapAppleMaps;
use Jtotal\Plugin\Content\Jtmap\Helper\JtmapGoogleMaps;
use Jtotal\Plugin\Content\Jtmap\Helper\JtmapHelper;

defined('_JEXEC') or die;

class Jtmap extends CMSPlugin
{
    /**
     * Application object
     *
     * @var    JApplicationCms
     * @since  3.7.0
     */
    protected $app;
    protected $jq_init;
    protected $autoloadLanguage = true;

    protected static $jquery_loaded = false;

    /**
     * Constructor
     *
     * @access      protected
     * @param       object  $subject The object to observe
     * @param       array   $config  An array that holds the plugin configuration
     * @since       1.0.0
     */
    public function __construct(&$subject, $config)
    {
        $this->loadLanguage();
        parent::__construct($subject, $config);

        //$doc = Factory::getDocument();
        //$wa = $doc->getWebAssetManager();
        //$wa->getRegistry()->addExtensionRegistryFile('plg_content_jtmap');

        //$wa->registerAndUseScript('mapkit.js', 'https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js', array('defer' => true, 'async' => true));
    }

    /**
     * PREPARE
     */
    public function onContentPrepareForm(Form $form, $data)
    {

        // Don't run if in the API Application
        // Don't run this plugin when the content is being indexed

        $app = Factory::getApplication();
        if ($app->isClient('api')) {
            return;
        }

        if (!($form instanceof Form)) {
            $this->_subject->setError('JERROR_NOT_A_FORM');
            return false;
        }

        $debug = $this->params->get('jtmap_debug', 0);
        $app = Factory::getApplication();

        //////////////////////////////
        //////////////////////////////
        //////////////////////////////
        // CHECK CONTEXT

        // Check if the field form is calling us
        $check_component = false;
        if (strpos($form->getName(), 'com_content.article') === 0) {
            $check_component = true;
        }

        if (strpos($form->getName(), 'com_content.articles.filter') === 0) {
            $check_component = false;
        }

        if (strpos($form->getName(), 'com_categories.categorycom_content') === 0) {
            $check_component = true;
        }

        if ($debug) {
            $app->enqueueMessage("JT maps: Form name: " . $form->getName() . ' | ' . json_encode($check_component), 'JTMAP - check name');
        }
        if (!$check_component) {
            return;
        }

        ////////////////////////////
        ////////  IN  EX  //////////
        ////////////////////////////

        /////CATEGORY
        $exists = false;

        if (!is_object($data)) {
            $data = (object) $data;
        }

        if (isset($data->catid)) {
            $cat_exc_inc = $this->params->get('cat_inc_exc', '1');
            $catids = $this->params->get('categories', array());

            $exists = $this->includeExcludeCategory($cat_exc_inc, $catids, $data->catid);
        }

        /////ARTICLES
        //TODO
        if ($debug) {
            $app->enqueueMessage("JTMAP: Included: " . json_encode($exists), 'JTMAP - check in/ex');
            $app->enqueueMessage("Check name: " . json_encode($check_component), 'JTMAP - check in/ex');
        }

        $input = Factory::getApplication()->input;
        $component = $input->get('option', '');

        //TODO FALANG
        if ($component == "com_falang") {
            return;
        }

        Form::addFormPath(JPATH_PLUGINS . '/' . $this->_type . '/' . $this->_name . '/forms');

        if (empty($data)) {
            $exists = true;
        }

        if ($exists == false && $this->params->get('jtmap_cat_inc_exc_show_message', '1') == 0) {
            if ($data->catid !== null) {
                return;
            }
        }

        if ($exists == false && !empty((array) $data)) {
            $form->loadFile('empty', false);
            return true;
        }

        $pluginParams = $this->params;

        $isAdministrator = Factory::getApplication()->isClient('administrator');


        switch ($form->getName()) {
            case "com_content.article":
                $form->loadFile('article', false);

                if ($data->attribs) {
                    $artParams = new Registry($data->attribs);
                } else {
                    $artParams = new Registry();
                }

                if ($pluginParams->get('jtmap_default_map', 'gmap') == 'gmap' || $isAdministrator) {
                    //require_once 'src/Helper/jtmapgooglemaps.php';
                    JtmapGoogleMaps::jtmapsGooglemapsForm($artParams, $pluginParams);

                } elseif ($pluginParams->get('jtmap_default_map', 'gmap') == 'amap') {
                    //require_once 'src/Helper/jtmapapplemaps.php';
                    JtmapAppleMaps::jtmapsApplemapsForm($artParams, $pluginParams);
                }
                //Get google maps
                //jtmapGoogleMaps::jtmapsGooglemapsForm();

                break;

            case "com_categories.categorycom_content":
                $cat_pluginParams = new Registry($this->params);
                if ($cat_pluginParams->get('jtmap_cat_on', '0') == 1) {
                    $form->loadFile('category', false);
                }

                break;
        }

        return true;
    }

    public function onContentAfterSave($context, $sql, $isNew)
    {

        /*         $newfile  = Path::clean(JPATH_PLUGINS . '/content/jtmap/assets/icons/test/onContentPrepareForm.txt');
        $content = 'onContentPrepareForm '. date("i:s");
        $fp = fopen($newfile,"wb");
        fwrite($fp, $content); */
            /*
        ob_start();
        var_dump($params);
        $result = ob_get_clean();

        $app = jFactory::getApplication();
        $app->enqueueMessage("address", 'JTdebug');
        $app->enqueueMessage($result, 'JTdebug'); */
    }

    public function onContentBeforeSave($context, &$article, $isNew, $data)
    {

        if ($context != 'com_content.article') {
            return true;
        }

        //coordinates and address from plugin item parameters
        $art_params = new Registry($article->attribs);

        $plugin_params = $this->params;

        if (!JtLicense::_checkLicense()) {
            $art_params->set('jtmap_trigger', 'default');
            //$art_params->set('jtmaps_address', '');
            $art_params->set('jtmap_advanced', '0');
            $art_params->set('show_info', '0');
            $art_params->set('jtmap_article_custom_params', '0');
        }

        $article->attribs = $art_params->toString();

        $address = $art_params->get('address', '');
        $latitude = $art_params->get('latitude', '');
        $longitude = $art_params->get('longitude', '');

        if (!$art_params) {
            return true;
        }

        return true;
    }

    public function onContentAfterDisplay($context, &$item, &$params)
    {
        $pluginParams = $this->params;

        $trigger = $params->get('jtmap_trigger', $pluginParams->get('jtmap_trigger', 'after_display'));
        if ($trigger == 'default') {
            $trigger = $pluginParams->get('jtmap_trigger', 'after_display');
        }

        if ($trigger != 'after_display') {
            return;
        }

        switch ($context) {
            case "com_content.article":
                return self::jtmapArticle($context, $item, $params);
                break;
        }
    }

    public function onContentAfterTitle($context, &$item, &$params)
    {
        $pluginParams = $this->params;

        $trigger = $params->get('jtmap_trigger', $pluginParams->get('jtmap_trigger', 'after_display'));
        if ($trigger == 'default') {
            $trigger = $pluginParams->get('jtmap_trigger', 'after_display');
        }

        if ($trigger != 'after_title') {
            return;
        }

        switch ($context) {
            case "com_content.article":
                return self::jtmapArticle($context, $item, $params);
                break;
        }
    }

    public function onContentBeforeDisplay($context, &$item, &$params)
    {

  
        $pluginParams = $this->params;

        $trigger = $params->get('jtmap_trigger', $pluginParams->get('jtmap_trigger', 'after_display'));
        if ($trigger == 'default') {
            $trigger = $pluginParams->get('jtmap_trigger', 'after_display');
        }

        if ($trigger != 'before_display') {
            return;
        }

        switch ($context) {
            case "com_content.article":
                return self::jtmapArticle($context, $item, $params);
                break;
        }
    }

    public function jtmapArticle($context, &$item, &$params)
    {

        if ($context != 'com_content.article') {
            return;
        }

        $pluginParams = $this->params;
        ////////////////////////////
        ////////  IN  EX  //////////
        ////////////////////////////
        $cat_exc_inc = $pluginParams->get('cat_inc_exc', '1');
        $catids = $pluginParams->get('categories', array());

        $exists = $this->includeExcludeCategory($cat_exc_inc, $catids, $item->catid);
        if (!$exists) {
            return;
        }

        $input = Factory::getApplication()->input;
        $view = $input->get('view', '');

        if ($view != 'article') {
            return;
        }

        if (!empty($this->params)) {
            $pluginParams = $this->params;
        } else {
            $pluginParams = $params;
        }

        $current_lang = Factory::getConfig()->get('language');
        $default_lang = ComponentHelper::getParams('com_languages')->get('site', 'en-GB');

        $artParams = $item->params;

        if ($artParams->get('jtmap_on', '0') == 0) {
            return;
        }

        //FALANG
        //el idioma por defecto es diferente, así que tengo que cargar los datos del idioma original
        if ($current_lang != $default_lang) {
            $db = Factory::getDbo();
            $query = $db->getQuery(true);

            $query->select('a.attribs')
                ->from($db->quoteName('#__content') . ' AS a')
                ->where('a.id = ' . (int) $item->id);

            // Load the filter data.
            $db->setQuery($query);

            try {
                $original_attribs = $db->loadObject();
            } catch (\RuntimeException $e) {
                return null;
            }

            $o_att = new Registry($original_attribs->attribs);

            if ($o_att->get('jtmap_on', '0') == '1') {
                $artParams->set('jtmap_on', $o_att->get('jtmap_on', '0'));
            }
            $artParams->set('jtmap_advanced', $o_att->get('jtmap_advanced', '0'));
            $artParams->set('jtmap_styles', $o_att->get('jtmap_styles', ''));
            $artParams->set('icon', $o_att->get('icon', 'map-marker-solid'));
            $artParams->set('custom_icon', $o_att->get('custom_icon', ''));
            $artParams->set('jtmap_icon_color', $o_att->get('jtmap_icon_color', '#000000'));
            $artParams->set('ico_w', $o_att->get('ico_w', '32'));
            $artParams->set('ico_h', $o_att->get('ico_h', '32'));
            $artParams->set('show_info', $o_att->get('show_info', '0'));
            $artParams->set('jtmap_description', $o_att->get('jtmap_description', ''));
            $artParams->set('jtmap_zoom', $o_att->get('jtmap_zoom', '12'));
            $artParams->set('latitude', $o_att->get('latitude', ''));
            $artParams->set('longitude', $o_att->get('longitude', ''));
        }

        $categories = Categories::getInstance("content", array("access" => false, "published" => 0));
        $jtmapcat_params = new Registry($categories->get($item->catid)->params);
        $jtmapcat_params_customs = new Registry($jtmapcat_params->get('jtmap_category_customs', array()));

        $strip_char_number = $pluginParams->get('strip_char', 300);

        $introtext = JtmapHelper::stripCharacter($strip_char_number, $pluginParams->get('strip_html', 1), $item);

        $images = new Registry($item->images);
        $itemimage = '';

        if ($images->get('image_intro', null) && File::exists(JPATH_SITE . '/' . $images->get('image_intro'))) {
            $itemimage = Uri::root(true) . "/" . $images->get('image_intro');
            $itemimage = '<img style="margin-right:6px;float:left;" src="' . $itemimage . '" alt="' . $item->title . '" />';
        }

        $address = str_replace("'", "\'", $artParams->get('jtmaps_address', ''));

        //plugin or category map?
        $use_map_cat = ($jtmapcat_params->get('jtmap_category_usecustoms', '0')) ?
        $jtmapcat_params_customs->get('jtmap_use_map', $pluginParams->get('jtmap_default_map', 'gmap')) :
        $pluginParams->get('jtmap_default_map', 'gmap');

        //category or article?
        $use_map = ($artParams->get('jtmap_article_custom_params', '0') == '1')
        ? $artParams->get('jtmap_use_map', $pluginParams->get('jtmap_default_map', 'gmap'))
        : $use_map_cat;


        if ($view == 'article' && ($address != '' || ($artParams->get('latitude', '') && $artParams->get('longitude', '')))) {

            $retorno = '<div class="jtmap_article_wrapper">';

            //GOGGLE MAPS
            if ($use_map == 'gmap') {
                //require_once 'src/Helper/jtmapgooglemaps.php';
                $retorno .= JtmapGoogleMaps::jtmapsGooglemapsArticle($pluginParams, $artParams, $item);
            } elseif ($use_map == 'amap') {
                //APPLE MAPS
                //require_once 'src/Helper/jtmapapplemaps.php';
                $retorno .= JtmapAppleMaps::jtmapsApplemapsArticle($pluginParams, $artParams, $item);
            } else {
                //GOOGLE MAPS (DEFAULT)
                //TODO OPENSTREEPMAPS
                //require_once 'src/Helper/jtmapgooglemaps.php';
                $retorno .= jtmapGoogleMaps::jtmapsGooglemapsArticle($pluginParams, $artParams, $item);
            }

            $lat = $artParams->get('latitude', $pluginParams->get('defaultlat', '41.29967392908295'));
            $lon = $artParams->get('longitude', $pluginParams->get('defaultlat', '-0.7455528733426681'));

            $buttonPos = 'justify-content-' . $pluginParams->get('show_openinmap_button_pos', 'start');

            $externalmap = '<div class="jtmap_showexternalbutton d-flex ' . $buttonPos . '">';
            $externalmap .= '<a href="https://maps.apple.com/?daddr=(' . $lat . ',' . $lon . ')&amp;dirflg=d" target="_blank" class=" btn  btn-success">
                        <i class="fa-solid fa-route"></i> ' . Text::_("JTMAP_GOTO_TEXT") . '
                    </a>';
            $externalmap .= '</div>';

            if ($pluginParams->get('show_openinmap_button', '1')) {
                $retorno .= $externalmap;
            }

            $retorno .= '</div>';
            return $retorno;
        }
    }


    //transition from OLD to NEW jtmaps
    public function onCategoryDisplay($context, $id)
    {
        return $this->jtmapCategory($context, $id);
    }

    public function jtmapCategory($context, $id)
    {
        $doc = Factory::getDocument();

        $pluginParams = $this->params;

        if ($context != 'category') {
            return;
        }

        if (!$id) {
            return;
        }


        $plugin_catParams = new Registry($this->params);

        if ($plugin_catParams->get('jtmap_cat_on', '0') == 0) {
            return;
        }


        $categories = Categories::getInstance("content", array("access" => false, "published" => 0));
        $jtmapcat_params = new Registry($categories->get($id)->params);

        if ($jtmapcat_params->get('jtmap_on', 0) == 0) {
            return;
        }

        ////////////////////////////
        ////////  IN  EX  //////////
        ////////////////////////////
        $cat_exc_inc = $pluginParams->get('cat_inc_exc', '1');
        $catids = $pluginParams->get('categories', array());

        $exists = $this->includeExcludeCategory($cat_exc_inc, $catids, $id);
        if (!$exists) {
            return;
        }

        $plugin_cat_params = new Registry($pluginParams->get('jtmap_category_params', array()));
        $cat_map_params = new Registry($jtmapcat_params->get('jtmap_category_customs', array()));

        $jtmap_use_map = $plugin_cat_params->get('jtmap_use_map', 'gmap');

        if ($jtmapcat_params->get('jtmap_category_usecustoms', '0') == '1') {
            $jtmap_use_map = $cat_map_params->get('jtmap_use_map', $plugin_cat_params->get('jtmap_use_map', 'gmap'));
        }

        if ($jtmap_use_map == 'gmap') {
            $output = JtmapGoogleMaps::jtmapsGooglemapsCategory($pluginParams, $jtmapcat_params, $id);
        } elseif ($jtmap_use_map == 'amap') {
            //TODO
            $output = JtmapAppleMaps::jtmapsApplemapsCategory($pluginParams, $jtmapcat_params, $id);
            //$output = JtmapAppleMaps::jtmapsApplemapsCategory($pluginParams, $jtmapcat_params, $id);
        } elseif ($jtmap_use_map == 'omap') {
            $output = JtmapGoogleMaps::jtmapsGooglemapsCategory($pluginParams, $jtmapcat_params, $id);
        }


        return $output;
    }

    private function includeExcludeCategory($cat_exc_inc, $catids, $currentCatId)
    {

        $exists = false;

        //INCLUDE
        if ($cat_exc_inc == "1" && !$exists) {
            if (is_array($catids)) {
                $exists = (in_array($currentCatId, $catids)) ? true : false;
            }
            if (empty($catids)) {
                $exists = true;
            } elseif ($catids[0] == "0" || $catids[0] == "-1") {
                $exists = true;
            }
            //EXCLUDE
        } else if ($cat_exc_inc == "0" && !$exists) {
            //INclude
            if (is_array($catids)) {
                $exists = (in_array($currentCatId, $catids)) ? false : true;
            }
            //all selected
            if (empty($catids) || $catids[0] == "-1") {
                $exists = true;
            } elseif ($catids[0] == "0") {
                $exists = false;
            }
        }

        return $exists;
    }
}
