<?php

/**
 * @package     JTMAP
 * @subpackage  jtmap
 *
 * @copyright   (C) 2023 jtotal
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */
namespace Jtotal\Plugin\Content\Jtmap\Fields;

defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Form\FormField;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Session\Session;
use Joomla\Database\ParameterType;

/**
 * Form Field class for the Joomla Platform.
 * Supports a generic list of options.
 *
 * @since  1.7.0
 */
class JtmapEmptyField extends FormField
{
    /**
     * The form field type.
     *
     * @var    string
     * @since  1.7.0
     */
    protected $type = 'JtmapEmpty';

    /**
     * Method to get the field input markup for a generic list.
     * Use the multiple attribute to enable multiselect.
     *
     * @return  string  The field input markup.
     *
     * @since   3.7.0
     */
    protected function getInput()
    {

        $categoryID = $this->form->getData()->get('catid');

        $categoryTitle = self::getCategoryName($categoryID);

        $modalId = $this->id;

        $db = Factory::getContainer()->get('DatabaseDriver');
        $extension_id_query = $db->setQuery("SELECT extension_id FROM #__extensions WHERE element = 'jtmap' AND folder = 'content'");
        $extension_id = $extension_id_query->loadObject()->extension_id;

        $urlEdit = 'index.php?option=com_plugins&amp;task=plugin.edit&amp;layout=modal&amp;extension_id=' . $extension_id . '&amp;' . Session::getFormToken() . '=1';

        $modal = '';
        $modal .= '<button'
        . ' class="btn bg-light"'
        . ' id="' . $this->id . '_edit"'
            . ' data-bs-toggle="modal"'
            . ' type="button"'
            . ' data-bs-target="#ModalEdit' . $modalId . '">'
            . '<i class="fas fa-cogs"></i>'
            . '</button>';

        $modal .= HTMLHelper::_(
            'bootstrap.renderModal',
            'ModalEdit' . $modalId,
            array(

                //'backdrop'    => 'static',
                'keyboard' => false,
                'closeButton' => false,
                'url' => $urlEdit,
                'height' => '400px',
                'width' => '800px',
                'bodyHeight' => 70,
                'modalWidth' => 80,
                'footer' => '<button type="button" class="btn btn-secondary" data-bs-dismiss="modal">'
                . Text::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</button>',
            )
        );

        $button = '';

        if ($this->form->getData()->get('catid') == null) {
            $button = '<div class="alert alert-primary alert-jtmap" role="alert">

                <h4 class="alert-heading"> ' . Text::_('PLG_JTMAP_CAT_EXCLUDED') . ': <span class="badge bg-secondary">' . $categoryTitle . '</span></h4>
            ' . Text::_('PLG_JTMAP_CAT_EXCLUDED_DESC') . ' ' .
                $modal . '
            </div>';
        } else {
            $button = '<div class="alert alert-primary alert-jtmap" role="alert">

                    <h4 class="alert-heading"> ' . Text::_('PLG_JTMAP_CAT_EXCLUDED') . ': <span class="badge bg-secondary">' . $categoryTitle . '</span></h4>
                ' . Text::_('PLG_JTMAP_CAT_EXCLUDED_DESC') . ' ' .
                $modal . '
                </div>';
        }

        return $button;
    }

    protected function getCategoryName($categoryId)
    {

        $land = Factory::getLanguage();
        $land->load('com_banners', JPATH_ADMINISTRATOR);

        if ($categoryId) {
            $db = $this->getDatabase();
            $query = $db->getQuery(true)
                ->select($db->quoteName('title'))
                ->from($db->quoteName('#__categories'))
                ->where($db->quoteName('id') . ' = :categoryId')
                ->bind(':categoryId', $categoryId, ParameterType::INTEGER);
            $db->setQuery($query);

            try {
                $name = $db->loadResult();
            } catch (\RuntimeException $e) {
                $this->setError($e->getMessage());
                return false;
            }

            return $name;
        }
        return Text::_('COM_BANNERS_NOCATEGORYNAME');
    }
}
