<?php

/**
 * @package     JTMAP
 * @subpackage  jtmap
 *
 * @copyright   (C) 2023 jtotal
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
*/

namespace Jtotal\Plugin\Content\Jtmap\Fields;

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;
use Joomla\CMS\Form\Field\ListField;
use Joomla\CMS\Session\Session;

defined('JPATH_PLATFORM') or die;

/**
 * Form Field class for the Joomla Platform.
 * Supports a generic list of options.
 *
 * @since  1.7.0
 */
class JtmapStylesField extends ListField
{
    /**
     * The form field type.
     *
     * @var    string
     * @since  1.7.0
     */
    protected $type = 'JtmapStyles';

    /**
     * Method to get the field input markup for a generic list.
     * Use the multiple attribute to enable multiselect.
     *
     * @return  string  The field input markup.
     *
     * @since   3.7.0
     */
    protected function getInput()
    {

        $html = '';

        //layout="joomla.form.field.list-fancy-select"

        $this->class = "form";

        $selected = $this->value;
        $options = (array) $this->getOptions();

        /** @var CMSApplication $app */
        $app = Factory::getApplication();
        $wa = $app->getDocument()->getWebAssetManager();

        $toJs = 'var jtmap_styles = {';

        foreach ($options as $key => $option) {
            $toJs .= $option->value . ': {';

                $toJs .= 'path: "' . $option->path . '",';
                //$toJs .= 'routecolor: "' . $option->routecolor . '"';
            $toJs .= '},';
        }

        $toJs .= '};';

        $wa->addInlineScript($toJs);


        
        $this->onchange = '
            document.getElementById(\'jtmap_miniicon\').src= jtcfroute_styles[this.value].path;
        ';


        $img = '<div class="thumb_wrapper">';

        foreach ($options as $key => $image) {
            $class = "";
            //if($image->value == '') continue;
            if ($selected == 0 && ($image->value == 'default' )) {
                $img .= '<img id="jtmap_miniicon" width="16" class="' . $class . '" src="' . $image->path . '" />';
            } elseif ($image->value == $selected) {
                $img .= '<img id="jtmap_miniicon" width="16" class="' . $class . '" src="' . $image->path . '" />';
            }
        }

      

        $img .= '</div>';

       //var_dump($img);

        $modalId = $this->id;

        $db = Factory::getContainer()->get('DatabaseDriver');
        $extension_id_query = $db->setQuery("SELECT extension_id FROM #__extensions WHERE element = 'jtmap' AND folder = 'content'");
        $extension_id = $extension_id_query->loadObject()->extension_id;
        
        $urlEdit = 'index.php?option=com_plugins&amp;task=plugin.edit&amp;layout=modal&amp;extension_id='.$extension_id.'&amp;' . Session::getFormToken() . '=1';
       
        $html .= '<div class="d-flex align-items-center">
            <span class="add-on mr2 btn bg-light">' . $img . '</span>
            ' . parent::getInput();


        $html .= '<button'
        . ' class="btn bg-light"'
        . ' id="' . $this->id . '_edit"'
        . ' data-bs-toggle="modal"'
        . ' type="button"'
        . ' data-bs-target="#ModalEdit' . $modalId . '">'
        . '<i class="fas fa-swatchbook"></i>'
        . '</button>';

        $html .= '</div>';



        $html .= HTMLHelper::_(
            'bootstrap.renderModal',
            'ModalEdit' . $modalId,
            array(
            
                //'backdrop'    => 'static',
                'keyboard'    => false,
                'closeButton' => false,
                'url'         => $urlEdit,
                'height'      => '400px',
                'width'       => '800px',
                'bodyHeight'  => 70,
                'modalWidth'  => 80,
                'footer'     => '<button type="button" class="btn btn-secondary" data-bs-dismiss="modal">'
                        . Text::_('JLIB_HTML_BEHAVIOR_CLOSE') . '</button>'
            )
        );

        return $html;
    }


    protected function getOptions()
    {

        $fieldname = preg_replace('/[^a-zA-Z0-9_\-]/', '_', $this->fieldname);

        //var_dump($this);
        $f_icon_root = URI::root();
        $f_icon_base = $f_icon_root . "media/plg_content_jtmap/icons/base/";
        $f_icon_path_color = $f_icon_root . "media/plg_content_jtmap/icons/custom/";



        //default
        $options   = array(
            (object) array(
                'value'    => 'default',
                'text'     =>  '---' . Text::_('JSELECT') . '---',
                'path'     =>  $f_icon_base . 'map-marker-solid.svg'
            )
        );

        $plugin = PluginHelper::getPlugin('content', 'jtmap');
        $plugin_params = new Registry($plugin->params);

        $default = (object) array(
            'style_title'       => 'default',
            'jtroutecolor'      =>  '#000000',
            'jtrouteicon'       =>  '',
            'jtrouteicon_size'  =>  '32'
        );


        foreach ($plugin_params->get('jtmap_styles', array()) as $key => $option) {
            //get icon path
            $path = '';
            //CUSTOM
            if ($option->custom_icon != '') {
                $path = $f_icon_root . $option->custom_icon;
            } else {
                $path = $f_icon_base . $option->icon . '.svg';
                if ($option->jtmap_icon_color != '#000000') {
                    $icon_color_name = ltrim($option->jtmap_icon_color, '#');
                    $path = $f_icon_path_color . $option->icon . '_' . $icon_color_name . '.svg';
                }
            }


            $value = (string) $key;
            $text  = trim((string) $option->style_title) != '' ? trim((string) $option->style_title) : $value;

            $tmp = array(
                'value'    => $value,
                'text'     =>  Text::alt($text, $fieldname),
                'path'     =>  $path
                //'disable'  => $disabled,
                //'class'    => (string) $option['class'],
                //'selected' => ($checked || $selected),
                //'checked'  => ($checked || $selected),
            );

            // Add the option object to the result set.
            $options[] = (object) $tmp;
        }

        reset($options);

        return $options;
    }
}
