<?php

/**
 * @package     Joomla.Plugin
 * @subpackage  System.Webauthn
 *
 * @copyright   (C) 2020 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Plugin\Content\Jtmap\Helper;

// Protect from unauthorized access
defined('_JEXEC') or die();

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;
use Jtotal\Jtfw\Helper\JtFunc;

class JtmapAppleMaps
{

    public static $jquery_loaded;

    /**
     * FORM
     *
     * @param [type] $plugins
     * @param [type] $params
     * @return void
     */
    public static function jtmapsApplemapsForm($artParams, $pluginParams)
    {

        $wa = Factory::getDocument()->getWebAssetManager();
        /** @var CMSApplication $app */
        $app = \Joomla\CMS\Factory::getApplication();

        $doc = $app->getDocument();

        $token = $pluginParams->get('amap_token', '');

        $lat = $artParams->get('latitude', null);
        $lon = $artParams->get('longitude', null);

        $def_zoom = $pluginParams->get('defaultzoom', 18);
        $zoom = $artParams->get('jtmap_zoom', $def_zoom);

        $equivalent_zoom = array(
            1 => 100,
            2 => 80,
            3 => 70,
            4 => 60,
            5 => 24,
            6 => 14,
            7 => 8,
            8 => 5,
            9 => 2,
            10 => 1,
            11 => 0.5,
            12 => 0.2,
            13 => 0.08,
            14 => 0.06,
            15 => 0.04,
            16 => 0.02,
            17 => 0.01,
            18 => 0.005,
            19 => 0.002,
            20 => 0.001,
            21 => 0.0005,
        );

        /******************** */
        /*ICONS*/
        /******************** */
        $f_icon_root = Uri::root();
        $f_icon_base = $f_icon_root . "media/plg_content_jtmap/icons/base/";

        $f_icon = JtmapHelper::getIcon($artParams);

        /*
        ob_start();
        var_dump( $f_icon);
        $result = ob_get_clean();

        $app = jFactory::getApplication();
        $app->enqueueMessage("address", 'JTdebug');
        $app->enqueueMessage($result, 'JTdebug');
         */

        $show_info = $artParams->get('show_info', 0);

        if (!$lat && !$lon) {
            $lat = $pluginParams->get('defaultlat', 42.63730753635921);
            $lon = $pluginParams->get('defaultlon', 0.0945958195499723);
        }

        $document = Factory::getDocument();

        $forcerload = ($pluginParams->get('jtmap_debug', 0)) ? '?forcereload=' . rand(0, 10000) : '';

        $map = "

        const mapkitToken = '" . $token . "';
        const jtLat = '" . $lat . "';
        const jtLon = '" . $lon . "';
        const mapkitZoom = '" . $equivalent_zoom[$zoom] . "';

        console.log(mapkitZoom);


        var f_icon_base = '" . $f_icon_base . "';
        var f_custom_icon_base = '" . $f_icon_root . "';



        function jtmaploadScriptMap(){

           document.querySelector('.jtmapshowmapbutton').innerHTML += ' <i class=\"fa-solid fas fa-spinner fa-spin-pulse fa-spin-reverse\"></i>';

           if('" . $token . "' == ''){
                document.querySelector('.jtmapshowmapbutton').textContent = 'Token not valid';
                return false;
            }

            loadScript('https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js')
                .then(() => loadScript('" . Uri::root() . "media/plg_content_jtmap/js/jtmapApplemapsForm.js" . $forcerload . "'))
                .then(() => {
                    document.querySelector('.jtmapshowmapbutton').classList.add('d-none');
                    document.querySelector('.jtmapshowmapbutton').classList.add('hide');
                    initAppleMap();
                })
            .catch(() => console.error('Something went wrong.'))

        }

        const loadScript = src => {
            return new Promise((resolve, reject) => {
              const script = document.createElement('script')
              script.type = 'text/javascript'
              script.onload = resolve
              script.onerror = reject
              script.src = src
              document.head.append(script)
            })
        }

        ";

        $doc->addCustomTag('<script type="text/javascript">' . $map . '</script>');
    }

    /**
     * jtmapsGooglemapsArticle
     *
     * @param [type] $plugins
     * @param [type] $params
     * @return html
     */
    public static function jtmapsApplemapsArticle($pluginParams, $artParams, $item)
    {

        $token = $pluginParams->get('amap_token', '');

        if ($token == '') {
            return;
        }

        $lat = $artParams->get('latitude', $pluginParams->get('defaultlat', '40.24606223287311'));
        $lon = $artParams->get('longitude', $pluginParams->get('defaultlat', '0.750112930702187'));

        $def_zoom = $pluginParams->get('defaultzoom', 18);
        $zoom = $artParams->get('jtmap_zoom', $def_zoom);

        $equivalent_zoom = array(
            1 => 100,
            2 => 80,
            3 => 70,
            4 => 60,
            5 => 24,
            6 => 14,
            7 => 8,
            8 => 5,
            9 => 2,
            10 => 1,
            11 => 0.5,
            12 => 0.2,
            13 => 0.08,
            14 => 0.06,
            15 => 0.04,
            16 => 0.02,
            17 => 0.01,
            18 => 0.005,
            19 => 0.002,
            20 => 0.001,
            21 => 0.0005,
        );

        $width = $pluginParams->get('defaultwidth');
        $height = $pluginParams->get('defaultheight');

        $tint = $pluginParams->get('amap_design_tint', '#000000');
        $colorScheme = $pluginParams->get('amap_design_colorscheme', 'light');

        $f_icon = JtmapHelper::getIcon($artParams);

        // Map type
        $maptype = $artParams->get('maptype', $pluginParams->get('maptype', 'roadmap'));

        //apple equivalent
        //default
        $amaptype = 'Standard';

        if ($maptype == 'satellite') {
            $amaptype = 'Satellite';
        } elseif ($maptype == 'hybrid') {
            $amaptype = 'Hybrid';
        } elseif ($maptype == 'terrain') {
            $amaptype = 'MutedStandard';
        }

        $document = Factory::getDocument();
        $loadObserveJS = '';

        //LOAD DEFAULT
        if ($pluginParams->get('jtmap_debug', 0) == 1 || $pluginParams->get('jtmap_loading', 'default') == 'default') {
            //$document->addscript('https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js');
         
            $doc = Factory::getDocument();
            $wa = $doc->getWebAssetManager();
            $wa->getRegistry()->addExtensionRegistryFile('plg_content_jtmap');
            $wa->useScript('mapkit.js');
 
        } else if ($pluginParams->get('jtmap_loading', 'default') == 'promise') {
            JtFunc::getPromiseScript(array('https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js'), 'jtmapAppleMapsInit()');
        } else if ($pluginParams->get('jtmap_loading', 'default') == 'observer') {

            $loadObserveJS = '

            function observeAndLoadMapkit(){
                const mapKitJsLoadedPromise = new Promise(resolve => {
                    const element = document.createElement("script");
                    element.addEventListener("load", resolve, { once : true });
                    element.src = "https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js"
                    element.setAttribute("data-callback","jtmapAppleMapsInit")
                    element.crossOrigin = "anonymous";
                    document.head.appendChild(element);
                });
            }

            const jtMap_observer_onIntersection = (entries, observer) => {
                for (const { isIntersecting, target } of entries) {
                if (isIntersecting) {
                    observeAndLoadMapkit();
                    observer.unobserve(target);
                }
                }
            };
            const jtMap_observer_options = {
                root: null,
                rootMargin: "50px 0px",
                threshold: [0]
              };
            const jtmap_canvas = document.getElementById("map_canvas");
            const jtMap_observer = new IntersectionObserver(jtMap_observer_onIntersection, jtMap_observer_options);

            jtMap_observer.observe(jtmap_canvas);';
        }

        $output = '
            <style type="text/css">

                #jtmap_container{display:block}
                #map_canvas{width:100%}

                #map_canvas{
                    width:100%;
                    height:' . $height . 'px;
                }
                #map_canvas img {
                    background: none !important;
                    max-width:none !important;
                }

            </style>

            <div id="map_canvas"></div>';

        $output .= "<script>";

        $lang = substr(Factory::getLanguage()->getTag(), 0, 2);

        $output .= $loadObserveJS;
        $output .= '

            function jtmapAppleMapsInit(){
                mapkit.init({
                    authorizationCallback: function(done) {
                        done("' . $token . '");
                    },
                    language: "' . $lang . '"
                });';

        if ($pluginParams->get('jtmap_debug', 0)) {
            $output .= 'console.log("jtmapAppleMapsInit");';
        }

        $output .= '

                const center = new mapkit.Coordinate(' . $lat . ',' . $lon . '),
                    span = new mapkit.CoordinateSpan(' . $equivalent_zoom[$zoom] . ', ' . $equivalent_zoom[$zoom] . '),
                    region = new mapkit.CoordinateRegion(center, span)
                ;

                var mapOptions = {
                    center : center,
                    region : region
                }
            ';

        //TINT
        $output .= 'mapOptions.tintColor = "' . $tint . '";';

        //COLOR SCHEME
        if ($colorScheme == 'auto') {
            $output .= '
                    const prefersColorSchemeDark = window.matchMedia(\'(prefers-color-scheme: dark)\');
                    if (prefersColorSchemeDark.matches) {
                        mapOptions.colorScheme = mapkit.Map.ColorSchemes.Dark;
                    }else{
                        mapOptions.colorScheme = mapkit.Map.ColorSchemes.Light;
                    }
                ';
        } elseif ($colorScheme == 'dark') {
            $output .= 'mapOptions.colorScheme = mapkit.Map.ColorSchemes.Dark;';
        } else {
            $output .= 'mapOptions.colorScheme = mapkit.Map.ColorSchemes.Light;';
        }

        //MAP CANVAS
        $output .= 'var map = new mapkit.Map("map_canvas", mapOptions);';

        //MAP MAPTYPE
        $output .= 'map.mapType = mapkit.Map.MapTypes.' . $amaptype . ';';

        //PDI annttation
        //TODO

        $infoMessage = '';
        if ($artParams->get('show_info', '0')) {
            $infoMessage = $artParams->get('jtmap_description', '');
        }
        /*
        if($artParams->get('jtmaps_address', '')){
            $infoMessage .= '<br><strong><i class=\"fas fa-map-pin\"></i> '.$artParams->get('jtmaps_address', '').'</strong>';
        }
        */

        //PDI IMAGE
        $output .= '

                var coordinate = new mapkit.Coordinate(' . $lat . ', ' . $lon . ');

                var pinOptions = {
                    title: "' . $item->title . '",
                    url: { 1: "' . $f_icon->get('icon_path', '') . '"},
                    selected: false
                };

                pinOptions.size = new Object({ width: ' . $f_icon->get('ico_w', '32') . ', height: ' . $f_icon->get('ico_h', '32') . ' });
                pinOptions.subtitle = "' . $infoMessage . '";
            ';

        if ($artParams->get('show_info', '0') == '1') {
            $output .= '
                    pinOptions.selected = true;
                ';
        }

        $output .= '
                var pinAnnotation = new mapkit.ImageAnnotation(coordinate, pinOptions);
                map.addAnnotation(pinAnnotation);

            ';

        $output .= '
            }';

        //DEBUG without promise
        if ($pluginParams->get('jtmap_debug', 0) == 1 || $pluginParams->get('jtmap_loading', 'default') == 'default') {
            $output .= '
            document.addEventListener("DOMContentLoaded", function(event) {
                jtmapAppleMapsInit();
            });
            ';
        }

        $output .= '</script>';

        return $output;
    }

    /**
     * jtmapsApplemapsCategory
     *
     * @param [type] $pluginParams
     * @param [type] $jtmapcat_params
     * @param [type] $id
     * @return string
     */
    public static function jtmapsApplemapsCategory($pluginParams, $jtmapcat_params, $id)
    {

        $token = $pluginParams->get('amap_token', '');

        if ($token == '') {
            return;
        }
 
        $input = Factory::getApplication()->input;
        $view = $input->get('view', '');

        $plugin_cat_params = new Registry($pluginParams->get('jtmap_category_params', array()));
        $cat_map_params = new Registry($jtmapcat_params->get('jtmap_category_customs', array()));

        //DEFAULTS
        $locale = $pluginParams->get('locale', 'en');
        $readmoreParam = $pluginParams->get('readmore', '0');

        $def_lat = $pluginParams->get('defaultlat', 42.63730753635921);
        $def_lon = $pluginParams->get('defaultlon', 0.0945958195499723);

        $cat_mapcontainer = $plugin_cat_params->get('map_container', '.content-category');
        $cat_maptype = $plugin_cat_params->get('maptype', 'roadmap');
        $cat_annotationtype = $plugin_cat_params->get('annotationtype', 'marker');
        $zoom = $plugin_cat_params->get('jtmap_zoom', '18');
        $catwidth = $plugin_cat_params->get('catwidth', '100%');
        $catheight = $plugin_cat_params->get('catheight', '400px');
        $catContainer = $plugin_cat_params->get('map_container', '.content-category');
        $map_insert_position = $plugin_cat_params->get('map_insert_position', 'after');

        //USE CUSTOMS from category
        if ($jtmapcat_params->get('jtmap_category_usecustoms', '0') == '1') {
            $cat_mapcontainer = $cat_map_params->get('map_container', $cat_mapcontainer);
            $cat_maptype = $cat_map_params->get('maptype', $cat_maptype);
            $cat_annotationtype = $cat_map_params->get('annotationtype', $cat_annotationtype);
            $zoom = $cat_map_params->get('jtmap_zoom', $zoom);
            $catwidth = $cat_map_params->get('catwidth', $catwidth);
            $catheight = $cat_map_params->get('catheight', $catheight);
            $catContainer = $cat_map_params->get('map_container', '.content-category');
            $map_insert_position = $cat_map_params->get('map_insert_position', 'after');
        }

        //var_dump($jtmapcat_params);


        $tint = $pluginParams->get('amap_design_tint', '#000000');
        $colorScheme = $pluginParams->get('amap_design_colorscheme', 'light');

        //apple equivalent
        //default
        $amaptype = 'Standard';

        if ($cat_maptype == 'satellite') {
            $amaptype = 'Satellite';
        } elseif ($cat_maptype == 'hybrid') {
            $amaptype = 'Hybrid';
        } elseif ($cat_maptype == 'terrain') {
            $amaptype = 'terrain';
        }

        //GET CATEGORY MODEL
        $app = Factory::getApplication();
        /** @var \Joomla\Component\Content\Site\Model\ArticlesModel $model */
        $model = $app->bootComponent('com_content')->getMVCFactory()->createModel('Articles', 'Site', ['ignore_request' => true]);
        // Set application parameters in model
        $appParams = $app->getParams();
        $model->setState('filter.access', true);
        $model->setState('params', $appParams);
        $model->setState('filter.category_id', $id);
        $items = $model->getItems();

        if (empty($items)) {
            return null;
        }

        $lats = array();
        $longs = array();
        $res = array();

        /**
         * BUCLE ARTICULOS
         */
        foreach ($items as $k => $item) {
            if (!$item->state) {
                continue;
            }

            $param = new Registry($item->attribs);
            $images = new Registry($item->images);

            //old map fix
            $long = ($param->get('longitude', null) == null) ? $param->get('lognitude', null) : $param->get('longitude', null);

            //LAT LONG FROM ADDRESS
            if (!$param->get('latitude', null) && !$long) {

                unset($items[$k]);
                continue;
            } else {
                $lat = (float) $param->get('latitude', null);
                $long = ($param->get('longitude', null) == null) ? $param->get('lognitude', null) : $param->get('longitude', null);
            }

            $items[$k]->lat = $lat;
            $lats[] = $lat;
            $items[$k]->long = $long;
            $longs[] = (float) $long;

            /*
            $pois[]  = array(
            'title' => $item->title,
            'lat' => (float) $lat,
            'long' => (float) $long,
            'icon' => HTMLHelper::cleanImageURL($f_icon->get('icon_path', ''))->url,
            'icon_w' => $f_icon->get('ico_w', '32'),
            'icon_h' => $f_icon->get('icon_h', '32'),
            'intro' => preg_replace("/\{.*\}/", '', $intro),
            'link' => $link1,
            'readmore' => $readmore,
            'route' => Route::_($link),
            'address_topoi' => $address_topoi
            ); */

            //var_dump($item->params);

            //FORCE CATEGORY
            if ($jtmapcat_params->get('force_cat_style', 0)) {
                $f_icon = JtmapHelper::getIcon($jtmapcat_params);
                $f_color = $jtmapcat_params->get('jtmap_icon_color', '#000000');
            } else {
                $f_icon = JtmapHelper::getIcon($param);
                $f_color = $param->get('jtmap_icon_color', '#000000');
            }

            $f_icon_url = ($f_icon->get('icon', 'map-marker-solid') == 'map-marker-solid') ? '' : HTMLHelper::cleanImageURL($f_icon->get('icon_path', ''))->url;

            $pois[] = array(
                'title' => $item->title,
                'lat' => (float) $lat,
                'long' => (float) $long,
                'category_alias' => $item->category_alias,
                'icon' => $f_icon_url,
                'icon_w' => $f_icon->get('ico_w', '32'),
                'icon_h' => $f_icon->get('icon_h', '32'),
                'fcolor' => $f_color,
                'link' => Route::_('index.php?option=com_content&view=article&id=' . $item->id . '&catid=' . $item->catid),
            );
        }

        $def_zoom = $pluginParams->get('defaultzoom', 18);
        $zoom = $pluginParams->get('jtmap_zoom', $def_zoom);

        $equivalent_zoom = array(
            1 => 100,
            2 => 80,
            3 => 70,
            4 => 60,
            5 => 24,
            6 => 14,
            7 => 8,
            8 => 5,
            9 => 2,
            10 => 1,
            11 => 0.5,
            12 => 0.2,
            13 => 0.08,
            14 => 0.06,
            15 => 0.04,
            16 => 0.02,
            17 => 0.01,
            18 => 0.005,
            19 => 0.002,
            20 => 0.001,
            21 => 0.0005,
        );

        //$f_icon = JtmapHelper::getIcon($artParams);

        $document = Factory::getDocument();
        $loadObserveJS = '';

        //LOAD DEFAULT
        if ($pluginParams->get('jtmap_debug', 0) == 1 || $pluginParams->get('jtmap_loading', 'default') == 'default') {
            //IN JTMAP SYSTEM

        } else if ($pluginParams->get('jtmap_loading', 'default') == 'promise') {
            JtFunc::getPromiseScript(array('https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js'), 'jtmapAppleMapsInit()');
        } else if ($pluginParams->get('jtmap_loading', 'default') == 'observer') {

            $loadObserveJS = '

            function observeAndLoadMapkit(){
                const mapKitJsLoadedPromise = new Promise(resolve => {
                    const element = document.createElement("script");
                    element.addEventListener("load", resolve, { once : true });
                    element.src = "https://cdn.apple-mapkit.com/mk/5.x.x/mapkit.js"
                    element.setAttribute("data-callback","jtmapAppleMapsInit")
                    element.crossOrigin = "anonymous";
                    document.head.appendChild(element);
                });
            }

            const jtMap_observer_onIntersection = (entries, observer) => {
                for (const { isIntersecting, target } of entries) {
                if (isIntersecting) {
                    observeAndLoadMapkit();
                    observer.unobserve(target);
                }
                }
            };
            const jtMap_observer_options = {
                root: null,
                rootMargin: "50px 0px",
                threshold: [0]
              };
            const jtmap_canvas = document.getElementById("map_canvas");
            const jtMap_observer = new IntersectionObserver(jtMap_observer_onIntersection, jtMap_observer_options);

            jtMap_observer.observe(jtmap_canvas);';
        }



        $output = '
        <style type="text/css">

            #jtmap_container{display:block}
            #map_canvas{width:100%}

            #map_canvas{
                width:' . $catwidth . ';
                height:' . $catheight . ';
            }
            #map_canvas img {
                background: none !important;
                max-width:none !important;
            }
            .circle-annotation {
                width: 32px;
                height: 32px;
                border-radius: 50%;
                color: #000;
                background-color: #fff;
                text-align: center;
                line-height: 32px;
            }
            .review-callout{
                background-color: #fff;
                padding: 10px;
                border-radius: 5px;
                box-shadow: 0 0 10px rgba(0,0,0,0.2);
            }
        </style>

        ';

        $output .= "<script>";

        $output .= '


        var parentDiv = document.querySelector("' . $cat_mapcontainer . '");
        var childDiv = document.createElement("div");
        childDiv.id = "map_canvas";
        parentDiv.appendChild(childDiv);';

        /*
        mapkit.init({
        authorizationCallback: function(done) {
        var xhr = new XMLHttpRequest();
        xhr.open("GET", "/services/jwt");
        xhr.addEventListener("load", function() {
        done(this.responseText);
        });
        xhr.send();
        }
        });
         */

        $lang = substr(Factory::getLanguage()->getTag(), 0, 2);

        $output .= $loadObserveJS;
        $output .= '

            var pois = \'' . json_encode($pois) . '\';


            const jtmaps_category_params = [];

            jtmaps_category_params.amaps_token = "' . $token . '";
            jtmaps_category_params.lang = "' . $lang . '";
            jtmaps_category_params.maptype = "' . $cat_maptype . '";
            jtmaps_category_params.tintColor = "' . $tint . '";

            jtmaps_category_params.def_lat = ' . $def_lat . ';
            jtmaps_category_params.def_lon = ' . $def_lon . ';
            jtmaps_category_params.def_zoom = ' . $equivalent_zoom[$zoom] . ';

            jtmaps_category_params.colorScheme = "' . $colorScheme . '";
            jtmaps_category_params.annotationType = "' . $cat_annotationtype . '";

            jtmaps_category_params.jtmap_container = "' . $catContainer . '";
            jtmaps_category_params.jtmap_insert_position = "' . $map_insert_position . '";

            //console.log(jtmaps_category_params);';


        //DEBUG without promise
        if ($pluginParams->get('jtmap_loading', 'default') == 'default') {
            $output .= '
            document.addEventListener("DOMContentLoaded", function(event) {
                jtmapAppleMapsInit();
            });
            ';
        }

        $output .= '</script>';

        //$output .= '<script src="' . Uri::root() . 'media/plg_content_jtmap/js/jtmap_amaps_category.js"></script>';

        return $output;
    }
}
