<?php

/**
 * @package     Joomla.Plugin
 * @subpackage  System.Webauthn
 *
 * @copyright   (C) 2020 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Plugin\Content\Jtmap\Helper;

// Protect from unauthorized access
defined('_JEXEC') or die();

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\Component\Content\Site\Helper\RouteHelper;
use Joomla\Registry\Registry;
use Jtotal\Jtfw\Helper\JtFunc;

class JtmapGoogleMaps
{

    public static $jquery_loaded;

    public static function jtmapsGooglemapsForm($artParams, $pluginParams)
    {

        $doc = Factory::getDocument();
        $wa = $doc->getWebAssetManager();
        $wa->getRegistry()->addExtensionRegistryFile('plg_content_jtmap');
        $wa->useScript('jtmap_gmaps_backend.js');


        $zoom = $artParams->get('jtmap_zoom', $pluginParams->get('defaultzoom', 18));

        $lat = $artParams->get('latitude', null);
        $lon = $artParams->get('longitude', null);

        $api = $pluginParams->get('gmap_api', '');

        /******************** */
        /*ICONS*/
        /******************** */
        $f_icon_root = Uri::root();
        $f_icon_base = $f_icon_root . "media/plg_content_jtmap/icons/base/";

        $f_icon = JtmapHelper::getIcon($artParams);

        /*
        ob_start();
        var_dump( $f_icon);
        $result = ob_get_clean();

        $app = jFactory::getApplication();
        $app->enqueueMessage("address", 'JTdebug');
        $app->enqueueMessage($result, 'JTdebug'); */

        $show_info = $pluginParams->get('show_info', 0);

        if (!$lat && !$lon) {
            $lat = $artParams->get('defaultlat', 41.2996709064967);
            $lon = $artParams->get('defaultlon', -0.7455528733426681);
        }

        $document = Factory::getDocument();

        //anton
        $estilo = '
            #attrib-jtmap img{
                max-width: inherit !important;
            }
            #gmap{
                width:100% !important;
                float:none;
            }
        ';

        $document->addStyleDeclaration($estilo);
            /*
        if (!self::$jquery_loaded) {
            $document = Factory::getDocument();
            HTMLHelper::_('jquery.framework');
            self::$jquery_loaded = true;
        }
        */

        $wa->addInlineScript('
   
            const jtmaps_gcat_params = [];
   
            jtmaps_gcat_params.gapi = "' . $api . '";         
            jtmaps_gcat_params.gapi_url = "//maps.googleapis.com/maps/api/js?key=' . $api.'";
            jtmaps_gcat_params.f_icon_base = "' . $f_icon_base . '";
            jtmaps_gcat_params.f_custom_icon_base = "' . $f_icon_root . '";
            jtmaps_gcat_params.lat_center = ' . $lat . ';
            jtmaps_gcat_params.long_center = ' . $lon . ';
            jtmaps_gcat_params.zoom = ' . $zoom . ';
            jtmaps_gcat_params.icon_path = "' . $f_icon->get('icon_path', 'https://developers.google.com/maps/documentation/javascript/images/default-marker.png') . '";
            jtmaps_gcat_params.icon_size_w = "' . $f_icon->get('ico_w', '32') . '";
            jtmaps_gcat_params.icon_size_h = "' . $f_icon->get('ico_h', '32') . '";

        ');

        $map = "
            var map;
            var f_icon_base = '" . $f_icon_base . "';
            var f_custom_icon_base = '" . $f_icon_root . "';
            var mapInitialized = false;

            function initMap() {
                if(mapInitialized) return;
                mapInitialized = true;
                var div = document.createElement('div');
                div.setAttribute('id','gmap');
                div.style.height = '400px';
                div.style.width = '100%';
                //jQuery(div).appendTo(jQuery('#jform_attribs_longitude').parent().parent().parent());

                const mapCanvas = document.getElementById('map_canvas');
                mapCanvas.appendChild(div);

                var centro = {lat: jtmaps_gcat_params.lat_center , lng: jtmaps_gcat_params.long_center};

                map = new google.maps.Map(document.getElementById('gmap'), {
                    center: centro,
                    zoom: tmaps_gcat_params.zoom
                });

                var icon = {
                    url: jtmaps_gcat_params.icon_path,
                    scaledSize: new google.maps.Size(jtmaps_gcat_params.icon_size_w, jtmaps_gcat_params.icon_size_h),
                }


                var marker = new google.maps.Marker({
                    position: centro,
                    map: map,
                    icon: icon,
                    draggable:true
                });

                marker.addListener('dragend', function(event) {

                    map.panTo(this.getPosition());

                    document.getElementById('jform_attribs_latitude').value = this.getPosition().lat();
                    document.getElementById('jform_attribs_longitude').value = this.getPosition().lng();

                });

                //CHANGE ZOOM
                jQuery('#jform_attribs_jtmap_zoom').on('change',function(){

                    map.setZoom(Number(jQuery('#jform_attribs_jtmap_zoom').val()));
                });

                jQuery('#jform_attribs_jtmap_zoom_range').on('change',function(){

                    map.setZoom(Number(jQuery('#jform_attribs_jtmap_zoom').val()));
                });


                //CHANGE ICON
                jQuery('#jform_attribs_icon').on('change',function(){
                    jtmapchange();
                });

                jQuery('#jform_attribs_ico_w,#jform_attribs_ico_h').on('change',function(){
                    jtmapchange();
                });

                jQuery('#jform_attribs_custom_icon').on('change',function(){
                    jtmapchange();
                });

                function jtmapchange(){

                    var v_icon = jQuery('#jform_attribs_icon').val();

                    if(v_icon == '') v_icon = 'map-marker-solid';

                    var v_custom_icon = jQuery('#jform_attribs_custom_icon').val();

                    var v_w = jQuery('#jform_attribs_ico_w').val();
                    var v_h = jQuery('#jform_attribs_ico_h').val();

                    if(v_icon == 'custom'){
                        icon.url = f_custom_icon_base+v_custom_icon;
                    }else{
                        icon.url = f_icon_base+v_icon+'.svg';
                    }

                    icon.scaledSize = new google.maps.Size(v_w,v_h);
                    marker.setIcon(icon);

                }
            }

        function jtmaploadScriptMap(){

            var url = jtmaps_gcat_params.gapi_url;

            var jqxhr = jQuery.ajax({
                url: url,
                cache: true,
                dataType: \"script\"
            }).done(function(e) {
                jQuery('.jtmapshowmapbutton').hide();
                initMap();
            }).fail(function() {
                jQuery('.jtmapshowmapbutton').html('Error');
            })
        }";

        //$document->addCustomTag('<script type="text/javascript" src="//maps.google.com/maps/api/js?sensor=false" ></script> <script type="text/javascript">'.$js.'</script>');

        //$document->addCustomTag('<script src="//maps.googleapis.com/maps/api/js?key='.$api.'&callback=initMap" async defer></script>');
        //$document->addCustomTag('<script type="text/javascript">' . $map . '</script>');
    }

    /**
     * jtmapsGooglemapsArticle
     *
     * @param [type] $plugins
     * @param [type] $params
     * @return html
     */
    public static function jtmapsGooglemapsArticle($pluginParams, $artParams, $item)
    {

        $api = $pluginParams->get('gmap_api', '');

        $lat = $artParams->get('latitude', '');
        $lon = $artParams->get('longitude', '');

        $def_zoom = $pluginParams->get('defaultzoom', 18);
        $zoom = $artParams->get('jtmap_zoom', $def_zoom);

        $locale = $pluginParams->get('locale', 'en');
        $width = $pluginParams->get('defaultwidth');
        $height = $pluginParams->get('defaultheight');
        $enable_direction = $pluginParams->get('direction', '0');
        $enable_clustering = $pluginParams->get('cluster', 'yes');
        $weatheritem = $pluginParams->get('weatheritem', '0');
        $clouditem = $pluginParams->get('clouditem', '0');
        $tempunit = $pluginParams->get('tempunit', 'CELSIUS');

        $address = str_replace("'", "\'", $artParams->get('address', ''));

        $f_icon = JtmapHelper::getIcon($artParams);

        // Map type
        $maptype = $artParams->get('maptype', $pluginParams->get('maptype', 'roadmap'));

        //apple equivalent
        //default
        $gmaptype = 'roadmap';

        if ($maptype == 'satellite') {
            $gmaptype = 'satellite';
        } elseif ($maptype == 'hybrid') {
            $gmaptype = 'hybrid';
        } elseif ($maptype == 'terrain') {
            $gmaptype = 'terrain';
        }

        $output = '
        <style type="text/css">

            #jtmap_container{display:block}
            #map_canvas{width:100%}

            #map_canvas {
                width:100%;
                height:' . $height . 'px;
            }
            #map_canvas img {
                background: none !important;
                max-width:none !important;
            }

        </style>

        <div id="map_canvas"></div>';

        $document = Factory::getDocument();
        $loadObserveJS = '';

        //LOAD DEFAULT
        if ($pluginParams->get('jtmap_debug', 0) == 1 || $pluginParams->get('jtmap_loading', 'default') == 'default') {
            $output .= '
                <script async defer type="text/javascript" src="//maps.google.com/maps/api/js?key=' . $api . '&callback=jtmapGoogleMapsInit&amp;language=' . $locale . '"></script>';
        } else if ($pluginParams->get('jtmap_loading', 'default') == 'promise') {
            JtFunc::getPromiseScript(array('//maps.google.com/maps/api/js?key=' . $api . '&language=' . $locale), 'jtmapGoogleMapsInit()');
        } else if ($pluginParams->get('jtmap_loading', 'default') == 'observer') {
            $loadObserveJS = '

            function observeAndLoadMapkit(){
                const mapKitJsLoadedPromise = new Promise(resolve => {
                    const element = document.createElement("script");
                    element.addEventListener("load", resolve, { once : true });
                    element.src = "//maps.google.com/maps/api/js?key=' . $api . '&callback=jtmapGoogleMapsInit&amp;language=' . $locale . '"
                    element.crossOrigin = "anonymous";
                    document.head.appendChild(element);
                });
            }

            const jtMap_observer_onIntersection = (entries, observer) => {
                for (const { isIntersecting, target } of entries) {
                if (isIntersecting) {
                    observeAndLoadMapkit();
                    observer.unobserve(target);
                }
                }
            };
            const jtMap_observer_options = {
                root: null,
                rootMargin: "50px 0px",
                threshold: [0]
              };
            const jtmap_canvas = document.getElementById("map_canvas");
            const jtMap_observer = new IntersectionObserver(jtMap_observer_onIntersection, jtMap_observer_options);

            jtMap_observer.observe(jtmap_canvas);';
        }

        $output .= "<script>";

        $output .= $loadObserveJS;

        $output .= "
            function jtmapGoogleMapsInit() {
            ";

        $infoMessage = '';
        if ($artParams->get('show_info', '0')) {
            $infoMessage = htmlspecialchars($artParams->get('jtmap_description', ''));
        }

        if ($artParams->get('jtmaps_address', '')) {
            $infoMessage .= '<br><strong><i class="fas fa-map-pin"></i> ' . $artParams->get('jtmaps_address', '') . '</strong>';
        }

        $output .= '
            //INFOWINDOW
            var infowindow = new google.maps.InfoWindow({

                content: \'<div id="jtmarker" class="jtmarker jtmarker_article">\'+
                            \'<h3 class="jtmarker_title">' . str_replace("'", "\'", $item->title) . '</h3>\'+
                            \'<div id="jtmap_content">\'+\'<span>' . $infoMessage . '</span>\'+\'</div>\'+' . '\'</div>\'
            });
        ';

        $output .= "
                    var centro = {lat: " . $lat . ", lng: " . $lon . "};
                    map = new google.maps.Map(document.getElementById('map_canvas'), {
                        center: centro,
                        zoom: " . $zoom . ",
                        mapTypeId: '" . $gmaptype . "'
                    });

                    var icon = {
                        url: '" . $f_icon->get('icon_path', '') . "',
                        scaledSize: new google.maps.Size(" . $f_icon->get('ico_w', '32') . ", " . $f_icon->get('ico_h', '32') . "),
                    }

                    var marker = new google.maps.Marker({
                        position: centro,
                        map: map,
                        icon: icon,
                        animation: google.maps.Animation.DROP,
                        title: '" . $item->title . "'
                    });";

        $output .= "
                    //CLICK
                    marker.addListener('click', function() {
                        infowindow.open(map, marker);
                    });
                ";

        if ($artParams->get('show_info', 0)) {
            $output .= "setTimeout(function(){
                    infowindow.open(map, marker);
                }, 1000);";
        }

        $output .= "}

        </script>";

        return $output;
    }

    public static function jtmapsGooglemapsCategory($pluginParams, $jtmapcat_params, $id)
    {
        $isAdministrator = Factory::getApplication()->isClient('administrator');
        if ($isAdministrator) {
            return;
        }

        $input = Factory::getApplication()->input;
        $view = $input->get('view', '');

        $plugin_cat_params = new Registry($pluginParams->get('jtmap_category_params', array()));
        $cat_map_params = new Registry($jtmapcat_params->get('jtmap_category_customs', array()));

        //DEFAULTS
        $locale = $pluginParams->get('locale', 'en');
        $readmoreParam = $pluginParams->get('readmore', '0');

        $cat_maptype = $plugin_cat_params->get('jtmap_cat_maptype', 'ROADMAP');
        $zoom = $plugin_cat_params->get('jtmap_zoom', '18');
        $catwidth = $plugin_cat_params->get('catwidth', '100%');
        $catheight = $plugin_cat_params->get('catheight', '400px');
        $catContainer = $plugin_cat_params->get('map_container', '.content-category');
        $map_insert_position = $plugin_cat_params->get('map_insert_position', 'after');

        //if($cat_params->)
        //USE CUSTOMS
        if ($jtmapcat_params->get('jtmap_category_usecustoms', '0') == '1') {
            $cat_maptype = $cat_map_params->get('maptype', $cat_maptype);
            $zoom = $cat_map_params->get('jtmap_zoom', $zoom);
            $catwidth = $cat_map_params->get('catwidth', $catwidth);
            $catheight = $cat_map_params->get('catheight', $catheight);
            $catContainer = $cat_map_params->get('map_container', '.content-category');
            $map_insert_position = $cat_map_params->get('map_insert_position', 'after');
        }

        $maxzoom = $pluginParams->get('maxzoom', '16');

        //redirect to item on marker click
        //$marker_redirect =  $pluginParams->get('marker_redirect', '0');

        /*
        //original
        if ($pluginParams->get('exclude', 0) == 1) {
        if (in_array($category->id, (array) $pluginParams->get('category_id', null)))
        return false;
        } */

        //GET CATEGORY MODEL
        $app = Factory::getApplication();
        /** @var \Joomla\Component\Content\Site\Model\ArticlesModel $model */
        $model = $app->bootComponent('com_content')->getMVCFactory()->createModel('Articles', 'Site', ['ignore_request' => true]);

        // Set application parameters in model
        $appParams = $app->getParams();
        $model->setState('filter.access', true);
        $model->setState('params', $appParams);
        $model->setState('filter.category_id', $id);
        $items = $model->getItems();

        if (empty($items)) {
            return null;
        }

        $lats = array();
        $longs = array();
        $res = array();

        foreach ($items as $k => $item) {

            if (!$item->state) {
                continue;
            }

            $param = new Registry($item->attribs);
            $images = new Registry($item->images);

            //old map fix
            $long = ($param->get('longitude', null) == null) ? $param->get('lognitude', null) : $param->get('longitude', null);

            //LAT LONG FROM ADDRESS
            if (!$param->get('latitude', null) && !$long) {

                unset($items[$k]);
                continue;
            } else {
                $lat = (float) $param->get('latitude', null);
                $long = ($param->get('longitude', null) == null) ? $param->get('lognitude', null) : $param->get('longitude', null);
            }

            if (!$lat || !$long) {
                continue;
            }

            $items[$k]->lat = $lat;
            $lats[] = $lat;
            $items[$k]->long = $long;
            $longs[] = (float) $long;

            ///////////////////
            //STRIP CHARACTER
            ///////////////////
            $strip_char_number = $pluginParams->get('strip_char', 300);
            $intro0 = JtmapHelper::stripCharacter($strip_char_number, $pluginParams->get('strip_html', 1), $item);

            ///////////////////
            //LINKS
            ///////////////////
            $item->slug = $item->alias ? ($item->id . ':' . $item->alias) : $item->id;
            $link = Route::_(RouteHelper::getArticleRoute($item->id, $item->catid));

            ///////////////////
            //IMAGE
            ///////////////////
            $linka = '';
            if ($pluginParams->get('jtmap_cat_image', 0)) {
                if ($images->get('image_intro', null) && File::exists(JPATH_SITE . '/' . $images->get('image_intro'))) {
                    $image_src = URI::root(true) . "/" . $images->get('image_intro');
                    $linka = '<a href="' . Route::_($link) . '"><img style="margin-right:10px;float:left; max-width:50%;" src="' . $image_src . '" alt="' . $item->title . '" /></a>';
                }
            }
            $intro = $linka;
            $intro .= $intro0;

            //readmore link
            if ($readmoreParam) {
                $readmore = "<a class='btn btn-success readmore' href='" . Route::_($link) . "'>" . Text::_('JTMAPS_READMORE') . "</a>";
            } else {
                $readmore = "";
            }

            //LINK
            $link1 = "<a href='" . Route::_($link) . "'>" . $item->title . "</a>";

            //ADDRESS
            $address_topoi = '';
            if ($param->get('jtmaps_address', '') != '') {
                //$address_topoi = '<p class="jtmap_address"><i class="fa fal fa-home"></i> ' . $param->get('jtmaps_address', '') . '</p>';
                $address_topoi = '<a href="https://maps.apple.com/?daddr=(' . $lat . ',' . $long . ')&amp;dirflg=d" target="_blank">
                <p class="jtmap_address"><i class="fa-solid fa-route"></i> ' . $param->get('jtmaps_address', '') . ' </p>
                </a>';
            }

            /******************** */
            /*ICONS*/
            /******************** */
            $icon = Uri::root() . "media/plg_content_jtmap/icons/" . $param->get('icon', 'map-marker-solid') . '.svg';
            $custom_icon = Uri::root() . $param->get('custom_icon', '');

            if ($param->get('icon', '') == 'custom') {
                $icon = Uri::root() . "media/plg_content_jtmap/icons/map-marker-solid.svg";
                $f_icon = ($param->get('custom_icon', '') == '' ? $icon : $custom_icon);
            } else {
                $f_icon = Uri::root() . "media/plg_content_jtmap/icons/" . $param->get('icon', 'map-marker-solid') . '.svg';
            }

            $f_icon_base = Uri::root() . "media/plg_content_jtmap/icons/";
            $f_custom_icon_base = Uri::root();

            $ico_w = $param->get('ico_w', '32');
            $ico_h = $param->get('ico_h', '32');

            $show_info = $param->get('show_info', 0);

            $ico_w = $param->get('ico_w', '32');
            $ico_h = $param->get('ico_h', '32');

            //FORCE CATEGORY
            if ($jtmapcat_params->get('force_cat_style', 0)) {
                $f_icon = JtmapHelper::getIcon($jtmapcat_params);
            } else {
                $f_icon = JtmapHelper::getIcon($param);
            }

            $pois[] = array(
                'title' => $item->title,
                'lat' => (float) $lat,
                'long' => (float) $long,
                'icon' => HTMLHelper::cleanImageURL($f_icon->get('icon_path', ''))->url,
                'icon_w' => $f_icon->get('ico_w', '32'),
                'icon_h' => $f_icon->get('icon_h', '32'),
                'intro' => preg_replace("/\{.*\}/", '', $intro),
                'link' => $link1,
                'readmore' => $readmore,
                'route' => Route::_($link),
                'address_topoi' => $address_topoi,
            );

            $iconsize1 = "32,32";
            $iconsize2 = "0,32";
        }

        if (empty($items)) {
            return;
        }

        if (empty($lats) || empty($longs)) {
            return false;
        }

        $lat_min = min($lats);
        $lat_max = max($lats);
        $long_min = min($longs);
        $long_max = max($longs);

        $lat_center = ($lat_min + $lat_max) / 2;
        $long_center = ($long_min + $long_max) / 2;

        $styledtext = Text::_('JTMAP_STYLED');

        $output = '<div id="jtmap_container" class="jtmap_container_category"><div id="map_canvas"></div>';

        $api = $pluginParams->get('gmap_api', '');

        $output .= '
            <style type="text/css">

                #jtmap_container{display:block}
                #map_canvas{width:100%}

                #map_canvas {
                    width: ' . $catwidth . ';
                    height: ' . $catheight . ';
                    overflow: hidden;
                    clear: both;
                }
            </style>';

        //variables

        $output .= '<script type="text/javascript">

            const jtmaps_gcat_params = [];

            jtmaps_gcat_params.markers = [];

            jtmaps_gcat_params.gapi = "' . $api . '";

            jtmaps_gcat_params.locale = "' . $locale . '";
            jtmaps_gcat_params.lat_center = "' . $lat_center . '";
            jtmaps_gcat_params.long_center = "' . $long_center . '";

            jtmaps_gcat_params.jtmapbase = "' . Uri::root() . 'plugins/content/jtmap/";
            jtmaps_gcat_params.jtmapmedia = "' . Uri::root() . 'media/plg_content_jtmap/";

            jtmaps_gcat_params.pois = ' . json_encode($pois) . ';

            jtmaps_gcat_params.jtmap_cat_type = "' . strtolower($cat_maptype) . '";

            jtmaps_gcat_params.jtmap_container = "' . $catContainer . '";
            jtmaps_gcat_params.jtmap_insert_position = "' . $map_insert_position . '";
        ';

        /*
        if ($view == 'category') {

        $layout = $input->get('layout', null);

        if ($layout == 'blog') {
        $output .= '
        jQuery("#jtmap_container").prependTo("' . $this->params->get('blog_selector', ".blog") . '");
        jQuery("#jtmap_container").css("display","block");';
        } else {
        $output .= '
        jQuery("#jtmap_container").prependTo("' . $this->params->get('category_selector', ".category-list") . '");
        jQuery("#jtmap_container").css("display","block");';
        }
        } else if ($view == 'categories') {

        $output .= '
        jQuery("#jtmap_container").prependTo("' . $this->params->get('categories_selector', ".categories-list") . '");
        jQuery("#jtmap_container").css("display","block");';
        }
         */

        $output .= '</script>';

        //script
        //$output .= '<script src="' . Uri::root() . 'media/plg_content_jtmap/js/markerclusterer.js"></script>';
        //$output .= '<script src="' . Uri::root() . 'media/plg_content_jtmap/js/jtmap_gmaps_category.js"></script>';
        /*
        //LOAD JQUERY IF DEFINED
        if ((int) $this->params->get('jquery', 0) == 1 && !self::$jq_init) {
        HTMLHelper::_('jquery.framework');
        self::$jq_init = true;
        }
         */

        return $output;
    }
}
