<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  System.Webauthn
 *
 * @copyright   (C) 2020 Open Source Matters, Inc. <https://www.joomla.org>
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Jtotal\Plugin\Content\Jtmap\Helper;

// Protect from unauthorized access
defined('_JEXEC') or die();

use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Filesystem\Path;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;

abstract class JtmapHelper
{

    /**
     * stripCharacter
     *
     * @param [type] $strip_char_number
     * @param [type] $strip
     * @param [type] $item
     * @return string
     */
    public static function stripCharacter($strip_char_number, $strip, $item)
    {
        $strip_char = $strip_char_number;
        if ($strip) {
            $intro = OutputFilter::cleanText($item->introtext);
            $intro = self::truncateIntro($intro, $strip_char);
        } else {
            $intro = $item->introtext;
        }

        return $intro;
    }

    /**
     * Undocumented function
     *
     * @param [type] $text
     * @param integer $chars
     * @return string
     */
    protected static function truncateIntro($text, $chars = 25)
    {
        if (strlen($text) <= $chars) {
            return $text;
        }
        $text = $text . " ";
        $text = substr($text, 0, $chars);
        $text = substr($text, 0, strrpos($text, ' '));
        $text = $text . "...";
        return $text;
    }

    /******************** */
    /*ICONS*/
    /******************** */
    /**
     * getIcon
     *
     * @param [type] $params
     * @param string $media_path
     * @return Object
     */
    public static function getIcon($params, $media_path = 'media/plg_content_jtmap/icons/')
    {

        $icon = $params->get('icon', 'map-marker-solid');

        //var_dump($icon);
        //PATHS
        $f_icon_root = Uri::root();
        $f_icon_base = $f_icon_root . $media_path . "base/";
        $def_icon = $f_icon_base . 'map-marker-solid.svg';
        $f_custom_icon_base = Uri::root();

        //check mode: simple | advanced
        $mode = ($params->get('jtmap_advanced', '0') ? 'advanced' : 'simple');

        /******************** */
        //SIMPLE - predefined icons and GET STYLES
        if ($mode == 'simple') {
            //SIMPLE -> use defined icons
            //TODO
            $f_icon = $def_icon;

            $selected_style = $params->get('jtmap_styles', 'default');

            $plugin = PluginHelper::getPlugin('content', 'jtmap');
            $plugin_params = new Registry($plugin->params);
            $plg_styles = (object) $plugin_params->get('jtmap_styles', (object) array());

            //ADD default

            $plg_styles->default = (object) array(
                'style_title' => 'default',
                'icon' => 'map-marker-solid',
                'jtmap_icon_color' => '#000000',
                'custom_icon' => '',
                'ico_w' => '32',
                'ico_h' => '32',
            );

            if (!isset($plg_styles->$selected_style)) {
                $selected_style = 'default';
            }

            $style_params = new Registry($plg_styles->$selected_style);

            //CHECKS
            //BITMAP image (custom)
            $icon_img = false;
            $icon_path = '';

            if ($style_params->get('custom_icon', '') != '') {
                $icon = $style_params->get('custom_icon', '');
                $icon_path = $f_icon_root . HTMLHelper::cleanImageURL($style_params->get('custom_icon', ''))->url;
                $icon_img = true;
            } else {
                $icon = $style_params->get('icon', 'map-marker-solid');
            }

            $icon_array = array(
                'icon_custom' => $icon_img,
                'icon' => HTMLHelper::cleanImageURL($icon)->url,
                'icon_path' => $icon_path,
                'icon_color' => $style_params->get('jtmap_icon_color', '#000000'),
                'ico_w' => $style_params->get('ico_w', '32'),
                'ico_h' => $style_params->get('ico_w', '32'),
            );
            $icon_params = new Registry($icon_array);

            $f_icon = self::writeIcon($icon_params, $media_path);
            if ($f_icon) {
                $icon_params->set('icon_path', $f_icon);
            }
        } else {
            /******************** */
            //ADVANCED -> use custom icons

            //CHECKS
            //BITMAP image (custom)
            $icon_img = false;
            $icon_path = '';
            if ($params->get('custom_icon', '') != '') {
                $icon = $params->get('custom_icon', '');
                $icon_path = $f_icon_root . $params->get('custom_icon', '');
                $icon_img = true;
            } else {
                $icon = $params->get('icon', 'map-marker-solid');
                if ($icon == 'custom') {
                    $icon = 'map-marker-solid';
                }
            }

            $icon_array = array(
                'icon_custom' => $icon_img,
                'icon' => $icon,
                'icon_path' => $icon_path,
                'icon_color' => $params->get('jtmap_icon_color', '#000000'),
                'ico_w' => $params->get('ico_w', '32'),
                'ico_h' => $params->get('ico_w', '32'),
            );

            $icon_params = new Registry($icon_array);

            $f_icon = self::writeIcon($icon_params, $media_path);
            if ($f_icon) {
                $icon_params->set('icon_path', $f_icon);
            }
        }

        return $icon_params;
    }

    /**
     * Undocumented function
     *
     * @param [type] $icon_params
     * @param string $media_path
     * @return string
     */
    private static function writeIcon($icon_params, $media_path = "media/plg_content_jtmap/icons/")
    {

        $f_icon_root = Uri::root();
        $icon = $icon_params->get('icon', 'map-marker-solid');
        //if custom, return
        if ($icon_params->get('icon_custom', false)) {
            return false;
        }

        $icon_color = $icon_params->get('icon_color', '#000000');

        if ($icon_color == '#000000') {
            $f_icon = $f_icon_root . $media_path . "base/" . $icon . '.svg';
            return $f_icon;
        }

        //check file
        $icon_color_name = ltrim($icon_color, '#');
        $basefile = Path::clean(JPATH_SITE . '/media/plg_content_jtmap/icons/base/' . $icon . '.svg');
        $newfile = Path::clean(JPATH_SITE . '/media/plg_content_jtmap/icons/custom/' . $icon . '_' . $icon_color_name . '.svg');

        if (!Folder::exists(JPATH_SITE . '/media/plg_content_jtmap/icons/custom')) {
            Folder::create(JPATH_SITE . '/media/plg_content_jtmap/icons/custom');
        }

        if (!file_exists($newfile)) {
            //write
            //converter to xml text
            $content = file_get_contents($basefile);
            //execute the replace, you can use reg exp if you need

            $content = str_replace('fill="currentColor"', 'fill="' . $icon_color . '"', $content);
            //construct a new xml based in new xml content
            //$svg = new SVGDocument( $xml );
            //make the output
            //$svg->output();
            $fp = fopen($newfile, "wb");
            fwrite($fp, $content);
        }

        $f_icon_custom = $f_icon_root . "media/plg_content_jtmap/icons/custom/" . $icon . '_' . $icon_color_name . '.svg';
        $f_icon = $f_icon_custom;

        return $f_icon;
    }
}
